@@ -45,7 +45,7 @@ array_t generic_red(
 //////////////////////////////////////////////////////////////
 // Call Cuda codes                                          //
 //////////////////////////////////////////////////////////////
-
+  py::gil_scoped_release release;
   array_t result = keops_binders::launch_keops< array_t, array_t, index_t >
           (tag1D2D,
            tagCpuGpu,
@@ -55,7 +55,7 @@ array_t generic_red(
            &args[0],
            nranges,
            &ranges[0]);
-  
+  py::gil_scoped_acquire acquire;
   return result;
 }
 
