@@ -1,9 +1,10 @@
 import importlib.util
 import os
 
-from pykeops import bin_folder, build_type
+import pykeops.config
 from pykeops.common.compile_routines import compile_specific_conv_routine
 from pykeops.common.utils import create_and_lock_build_folder
+from pykeops.common.set_path import set_build_folder
 from pykeops.numpy import default_dtype
 
 
@@ -18,13 +19,13 @@ class LoadKeopsSpecific:
         
         spec = importlib.util.find_spec(dllname)
         
-        if (spec is None) or (build_type == 'Debug'):
-            self.build_folder = bin_folder + os.path.sep + 'build-' + self.dll_name
+        if (spec is None) or (pykeops.config.build_type == 'Debug'):
+            self.build_folder = set_build_folder(pykeops.config.bin_folder, self.dll_name)
             self._safe_compile()
     
     @create_and_lock_build_folder()
     def _safe_compile(self):
-        compile_specific_conv_routine(self.dll_name, self.dtype, build_folder=self.build_folder)
+        compile_specific_conv_routine(self.dll_name, self.dtype, self.build_folder)
     
     def import_module(self):
         return importlib.import_module(self.dll_name)
