@@ -8,6 +8,7 @@ import itertools
 import numpy as np
 
 import pykeops
+import pykeops.config
 from pykeops.numpy.utils import squared_distances, np_kernel, log_np_kernel, grad_np_kernel, differences, log_sum_exp
 
 
@@ -44,35 +45,35 @@ class PytorchUnitTestCase(unittest.TestCase):
         
         dtype = torch.float32
         xc = torch.tensor(x, dtype=dtype, device=device, requires_grad=True)
-        ac = torch.tensor(a, dtype=dtype, device=device, requires_grad=True)
-        ec = torch.tensor(e, dtype=dtype, device=device, requires_grad=True)
+        ac = torch.tensor(a, dtype=dtype, device=device, requires_grad=False)
+        ec = torch.tensor(e, dtype=dtype, device=device, requires_grad=False)
         fc = torch.tensor(f, dtype=dtype, device=device, requires_grad=True)
-        yc = torch.tensor(y, dtype=dtype, device=device, requires_grad=True)
-        bc = torch.tensor(b, dtype=dtype, device=device, requires_grad=True)
+        yc = torch.tensor(y, dtype=dtype, device=device, requires_grad=False)
+        bc = torch.tensor(b, dtype=dtype, device=device, requires_grad=False)
         gc = torch.tensor(g, dtype=dtype, device=device, requires_grad=True)
-        pc = torch.tensor(p, dtype=dtype, device=device, requires_grad=True)
+        pc = torch.tensor(p, dtype=dtype, device=device, requires_grad=False)
         sigmac = torch.tensor(sigma, dtype=dtype, device=device, requires_grad=False)
         alphac = torch.tensor(alpha, dtype=dtype, device=device, requires_grad=False)
 
         Xc = torch.tensor(X, dtype=dtype, device=device, requires_grad=True)
-        Lc = torch.tensor(L, dtype=dtype, device=device, requires_grad=True)
+        Lc = torch.tensor(L, dtype=dtype, device=device, requires_grad=False)
         Yc = torch.tensor(Y, dtype=dtype, device=device, requires_grad=True)
         Sc = torch.tensor(S, dtype=dtype, device=device, requires_grad=True)
 
         
         dtype = torch.float64
-        xcd = torch.tensor(x, dtype=dtype, device=device, requires_grad=True)
-        acd = torch.tensor(a, dtype=dtype, device=device, requires_grad=True)
-        ecd = torch.tensor(e, dtype=dtype, device=device, requires_grad=True)
-        fcd = torch.tensor(f, dtype=dtype, device=device, requires_grad=True)
-        ycd = torch.tensor(y, dtype=dtype, device=device, requires_grad=True)
-        bcd = torch.tensor(b, dtype=dtype, device=device, requires_grad=True)
-        gcd = torch.tensor(g, dtype=dtype, device=device, requires_grad=True)
-        pcd = torch.tensor(p, dtype=dtype, device=device, requires_grad=True)
+        xcd = torch.tensor(x, dtype=dtype, device=device, requires_grad=False)
+        acd = torch.tensor(a, dtype=dtype, device=device, requires_grad=False)
+        ecd = torch.tensor(e, dtype=dtype, device=device, requires_grad=False)
+        fcd = torch.tensor(f, dtype=dtype, device=device, requires_grad=False)
+        ycd = torch.tensor(y, dtype=dtype, device=device, requires_grad=False)
+        bcd = torch.tensor(b, dtype=dtype, device=device, requires_grad=False)
+        gcd = torch.tensor(g, dtype=dtype, device=device, requires_grad=False)
+        pcd = torch.tensor(p, dtype=dtype, device=device, requires_grad=False)
         sigmacd = torch.tensor(sigma, dtype=dtype, device=device, requires_grad=False)
         alphacd = torch.tensor(alpha, dtype=dtype, device=device, requires_grad=False)
         Xcd = torch.tensor(X, dtype=dtype, device=device, requires_grad=True)
-        Lcd = torch.tensor(L, dtype=dtype, device=device, requires_grad=True)
+        Lcd = torch.tensor(L, dtype=dtype, device=device, requires_grad=False)
         Ycd = torch.tensor(Y, dtype=dtype, device=device, requires_grad=True)
         Scd = torch.tensor(S, dtype=dtype, device=device, requires_grad=True)
         
@@ -81,70 +82,14 @@ class PytorchUnitTestCase(unittest.TestCase):
         print('Pytorch could not be loaded. Skip tests.')
         pass
     
-    ############################################################
-    def test_conv_kernels_feature(self):
-    ############################################################
-        from pykeops.torch.kernel_product.kernels import Kernel, kernel_product
-        params = {
-            'gamma': 1. / self.sigmac ** 2,
-            'mode': 'sum',
-        }
-        if pykeops.gpu_available:
-            backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'pytorch']
-        else:
-            backend_to_test = ['auto', 'pytorch']
-        
-        for k, b in itertools.product(['gaussian', 'laplacian', 'cauchy', 'inverse_multiquadric'], backend_to_test):
-            with self.subTest(k=k, b=b):
-                params['id'] = Kernel(k + '(x,y)')
-                params['backend'] = b
-                # Call cuda kernel
-                gamma = kernel_product(params, self.xc, self.yc, self.bc).cpu()
-                
-                # Numpy version
-                gamma_py = np.matmul(np_kernel(self.x, self.y, self.sigma, kernel=k), self.b)
-                
-                # compare output
-                self.assertTrue(np.allclose(gamma.cpu().data.numpy(), gamma_py))
-    
-    ############################################################
-    def test_grad1conv_kernels_feature(self):
-    ############################################################
-        import torch
-        from pykeops.torch import Kernel, kernel_product
 
-        params = {
-            'gamma': 1. / self.sigmac ** 2,
-            'mode': 'sum',
-        }
-        if pykeops.gpu_available:
-            backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'pytorch']
-        else:
-            backend_to_test = ['auto', 'pytorch']
-        
-        for k, b in itertools.product(['gaussian', 'laplacian', 'cauchy', 'inverse_multiquadric'], backend_to_test):
-            with self.subTest(k=k, b=b):
-                params['id'] = Kernel(k + '(x,y)')
-                params['backend'] = b
-                
-                # Call cuda kernel
-                aKxy_b = torch.dot(self.ac.view(-1), kernel_product(params, self.xc, self.yc, self.bc).view(-1))
-                gamma_keops = torch.autograd.grad(aKxy_b, self.xc, create_graph=False)[0].cpu()
-                
-                # Numpy version
-                A = differences(self.x, self.y) * grad_np_kernel(self.x, self.y, self.sigma, kernel=k)
-                gamma_py = 2 * (np.sum(self.a * (np.matmul(A, self.b)), axis=2)).T
-                
-                # compare output
-                self.assertTrue(np.allclose(gamma_keops.cpu().data.numpy(), gamma_py, atol=1e-6))
-    
     ############################################################
     def test_generic_syntax_float(self):
     ############################################################
         from pykeops.torch import Genred
         aliases = ['p=Pm(1)', 'a=Vj(1)', 'x=Vi(3)', 'y=Vj(3)']
         formula = 'Square(p-a)*Exp(x+y)'
-        if pykeops.gpu_available:
+        if pykeops.config.gpu_available:
             backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'GPU']
         else:
             backend_to_test = ['auto']
@@ -165,7 +110,7 @@ class PytorchUnitTestCase(unittest.TestCase):
         from pykeops.torch import Genred
         aliases = ['p=Pm(1)', 'a=Vj(1)', 'x=Vi(3)', 'y=Vj(3)']
         formula = 'Square(p-a)*Exp(x+y)'
-        if pykeops.gpu_available:
+        if pykeops.config.gpu_available:
             backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'GPU']
         else:
             backend_to_test = ['auto']
@@ -187,7 +132,7 @@ class PytorchUnitTestCase(unittest.TestCase):
         aliases = ['p=Pm(1)', 'a=Vj(1)', 'x=Vi(3)', 'y=Vj(3)']
         formula = 'Square(p-a)*Exp(-SqNorm2(x-y))'
         formula_weights = 'y'
-        if pykeops.gpu_available:
+        if pykeops.config.gpu_available:
             backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'GPU']
         else:
             backend_to_test = ['auto']
@@ -217,7 +162,7 @@ class PytorchUnitTestCase(unittest.TestCase):
         
         formula = 'Pow((X|Y),2) * ((Elem(P,0) * X) + (Elem(P,1) * Y))'
         
-        if pykeops.gpu_available:
+        if pykeops.config.gpu_available:
             backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'GPU']
         else:
             backend_to_test = ['auto']
@@ -237,20 +182,20 @@ class PytorchUnitTestCase(unittest.TestCase):
     ############################################################
     def test_logSumExp_kernels_feature(self):
     ############################################################
-        from pykeops.torch import Kernel, kernel_product
+        from pykeops.torch import Vi, Vj, Pm
 
-        params = {'gamma': 1. / self.sigmac ** 2, 'mode': 'lse'}
-        if pykeops.gpu_available:
-            backend_to_test = ['auto', 'GPU_1D', 'GPU_2D', 'pytorch']
-        else:
-            backend_to_test = ['auto', 'pytorch']
+        kernels = {'gaussian'   : lambda xc, yc, sigmac : (- Pm(1 / sigmac ** 2)   * Vi(xc).sqdist(Vj(yc)) ),
+                   'laplacian'  : lambda xc, yc, sigmac : (- (Pm(1 / sigmac ** 2)  * Vi(xc).sqdist(Vj(yc)) ).sqrt()),
+                   'cauchy'     : lambda xc, yc, sigmac : (1 + Pm(1 / sigmac ** 2) * Vi(xc).sqdist(Vj(yc)) ).power(-1).log(),
+                   'inverse_multiquadric'  : lambda xc, yc, sigmac : (1 + Pm(1 / sigmac ** 2) * Vi(xc).sqdist(Vj(yc)) ).sqrt().power(-1).log()
+        }
         
-        for k, b in itertools.product(['gaussian', 'laplacian', 'cauchy', 'inverse_multiquadric'], backend_to_test):
-            with self.subTest(k=k, b=b):
-                params['id'] = Kernel(k + '(x,y)')
-                params['backend'] = b
+        for k in ['gaussian', 'laplacian', 'cauchy', 'inverse_multiquadric']:
+            with self.subTest(k=k):
                 # Call cuda kernel
-                gamma = kernel_product(params, self.xc, self.yc, self.gc).cpu()
+                gamma_lazy = kernels[k](self.xc, self.yc, self.sigmac)
+                gamma_lazy = gamma_lazy.logsumexp(dim=1, weight=Vj(self.gc.exp())).cpu()
+                #gamma = kernel_product(params, self.xc, self.yc, self.gc).cpu()
                 
                 # Numpy version
                 log_K = log_np_kernel(self.x, self.y, self.sigma, kernel=k)
@@ -258,7 +203,7 @@ class PytorchUnitTestCase(unittest.TestCase):
                 gamma_py = log_sum_exp(log_KP, axis=1)
                 
                 # compare output
-                self.assertTrue(np.allclose(gamma.data.numpy().ravel(), gamma_py, atol=1e-6))
+                self.assertTrue(np.allclose(gamma_lazy.data.numpy().ravel(), gamma_py, atol=1e-6))
     
     ############################################################
     def test_logSumExp_gradient_kernels_feature(self):
@@ -400,7 +345,7 @@ class PytorchUnitTestCase(unittest.TestCase):
     
     ############################################################
     def test_LazyTensor_sum(self):
-        ############################################################
+    ############################################################
         import torch
         from pykeops.torch import LazyTensor
         
@@ -440,7 +385,7 @@ class PytorchUnitTestCase(unittest.TestCase):
     
     ############################################################
     def test_LazyTensor_logsumexp(self):
-        ############################################################
+    ############################################################
         import torch
         from pykeops.torch import LazyTensor
         
@@ -465,10 +410,15 @@ class PytorchUnitTestCase(unittest.TestCase):
                 a_i = K_ij.logsumexp(self.nbatchdims + 1)
                 if use_keops: a_i = a_i.squeeze(-1)
                 [g_x, g_y, g_s] = torch.autograd.grad((1. * a_i).sum(), [x, y, s], create_graph=True)
-                [g_xs] = torch.autograd.grad((g_x.abs()).sum(), [s], create_graph=True)
                 
-                results += [a_i, g_x, g_y, g_s, g_xs]
-            
+                # N.B. (Joan, sept 2020) commenting out the 2nd order gradient computation here,
+                # since it slows down too much the compilation currently, when using Cuda 11.
+                #
+                #[g_xs] = torch.autograd.grad((g_x.abs()).sum(), [s], create_graph=True)
+                #results += [a_i, g_x, g_y, g_s, g_xs]
+
+                results += [a_i, g_x, g_y, g_s]
+
             full_results.append(results)
         
         for (res_keops, res_torch) in zip(full_results[0], full_results[1]):
