@@ -5,7 +5,7 @@ from pykeops.common.keops_io import LoadKeOps
 from pykeops.common.operations import preprocess, postprocess
 from pykeops.torch.half2_convert import preprocess_half2, postprocess_half2
 from pykeops.common.parse_type import get_type, get_sizes, complete_aliases
-from pykeops.common.parse_type import get_accuracy_flags
+from pykeops.common.parse_type import get_optional_flags
 from pykeops.common.utils import axis2cat
 from pykeops.torch import default_dtype, include_dirs
 
@@ -15,12 +15,19 @@ class GenredAutograd(torch.autograd.Function):
     """
 
     @staticmethod
-    def forward(ctx, formula, aliases, backend, dtype, device_id, ranges, accuracy_flags, *args):
-    
-        optional_flags = include_dirs + accuracy_flags
-
-        myconv = LoadKeOps(formula, aliases, dtype, 'torch', optional_flags).import_module()
+    def forward(ctx, formula, aliases, backend, dtype, device_id, ranges, optional_flags, rec_multVar_highdim, *args):
+        
+        # N.B. when rec_multVar_highdim option is set, it means that formula is of the form "sum(F*b)", where b is a variable
+        # with large dimension. In this case we set compiler option MULT_VAR_HIGHDIM to allow for the use of the special "final chunk" computation
+        # mode. However, this may not be also true for the gradients of the same formula. In fact only the gradient
+        # with respect to variable b will have the same form. Hence, we save optional_flags current status into ctx, 
+        # before adding the MULT_VAR_HIGHDIM compiler option.
+        ctx.optional_flags = optional_flags.copy()
+        if rec_multVar_highdim is not None:
+            optional_flags += ['-DMULT_VAR_HIGHDIM=1']
 
+        myconv = LoadKeOps(formula, aliases, dtype, 'torch', optional_flags+include_dirs).import_module()
+        
         # Context variables: save everything to compute the gradient:
         ctx.formula = formula
         ctx.aliases = aliases
@@ -28,7 +35,7 @@ class GenredAutograd(torch.autograd.Function):
         ctx.dtype = dtype
         ctx.device_id = device_id
         ctx.ranges = ranges
-        ctx.accuracy_flags = accuracy_flags
+        ctx.rec_multVar_highdim = rec_multVar_highdim
         ctx.myconv = myconv
 
         tagCPUGPU, tag1D2D, tagHostDevice = get_tag_backend(backend, args)
@@ -41,6 +48,9 @@ class GenredAutograd(torch.autograd.Function):
         
         if ranges is None : ranges = () # To keep the same type
 
+        # N.B.: KeOps C++ expects contiguous integer arrays as ranges
+        ranges = tuple(r.contiguous() for r in ranges)
+
         result = myconv.genred_pytorch(tagCPUGPU, tag1D2D, tagHostDevice, device_id, ranges, *args)
 
         # relying on the 'ctx.saved_variables' attribute is necessary  if you want to be able to differentiate the output
@@ -56,7 +66,7 @@ class GenredAutograd(torch.autograd.Function):
         backend = ctx.backend
         dtype = ctx.dtype
         ranges = ctx.ranges
-        accuracy_flags = ctx.accuracy_flags
+        optional_flags = ctx.optional_flags
         device_id = ctx.device_id
         myconv = ctx.myconv
         args = ctx.saved_tensors[:-1]  # Unwrap the saved variables
@@ -89,11 +99,11 @@ class GenredAutograd(torch.autograd.Function):
 
         for (var_ind, (sig, arg_ind)) in enumerate(zip(aliases, args)):  # Run through the arguments
             # If the current gradient is to be discarded immediatly...
-            if not ctx.needs_input_grad[var_ind + 7]:  # because of (formula, aliases, backend, dtype, device_id, ranges, accuracy_flags)
+            if not ctx.needs_input_grad[var_ind + 8]:  # because of (formula, aliases, backend, dtype, device_id, ranges, optional_flags, rec_multVar_highdim)
                 grads.append(None)  # Don't waste time computing it.
 
-            else:  # Otherwise, the current gradient is really needed by the user:
-                # adding new aliases is way too dangerous if we want to compute
+            else:  
+                # Otherwise, the current gradient is really needed by the user. Adding new aliases is way too dangerous if we want to compute
                 # second derivatives, etc. So we make explicit references to Var<ind,dim,cat> instead.
                 # New here (Joan) : we still add the new variables to the list of "aliases" (without
                 # giving new aliases for them) these will not be used in the C++ code,
@@ -107,10 +117,18 @@ class GenredAutograd(torch.autograd.Function):
                 # N.B.: if I understand PyTorch's doc, we should redefine this function every time we use it?
                 genconv = GenredAutograd().apply
 
+                # For a reduction of the type sum(F*b), with b a variable, and if we require the gradient
+                # with respect to b, the gradient will be of same type sum(F*eta). So we set again rec_multVar option
+                # in this case.
+                if pos==ctx.rec_multVar_highdim:
+                    rec_multVar_highdim = nargs # nargs is the position of variable eta.
+                else:
+                    rec_multVar_highdim = None
+
                 if cat == 2:  # we're referring to a parameter, so we'll have to sum both wrt 'i' and 'j'
                     # WARNING !! : here we rely on the implementation of DiffT in files in folder keops/core/formulas/reductions
                     # if tagI==cat of V is 2, then reduction is done wrt j, so we need to further sum output wrt i
-                    grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, accuracy_flags, *args_g)
+                    grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, optional_flags, rec_multVar_highdim, *args_g)
                     # Then, sum 'grad' wrt 'i' :
                     # I think that '.sum''s backward introduces non-contiguous arrays,
                     # and is thus non-compatible with GenredAutograd: grad = grad.sum(0)
@@ -122,7 +140,7 @@ class GenredAutograd(torch.autograd.Function):
                         i for (i, (x, y)) in enumerate(zip(arg_ind.shape[:-1], grad.shape[:-1])) if x < y)
 
                 else:
-                    grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, accuracy_flags, *args_g)
+                    grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, optional_flags, rec_multVar_highdim, *args_g)
 
                     # N.B.: 'grad' is always a full [A, .., B, M, D] or [A, .., B, N, D] or [A, .., B, D] tensor,
                     #       whereas 'arg_ind' may have some broadcasted batched dimensions.
@@ -137,8 +155,8 @@ class GenredAutograd(torch.autograd.Function):
                 grad = grad.reshape(arg_ind.shape)  # The gradient should have the same shape as the input!
                 grads.append(grad)
         
-        # Grads wrt. formula, aliases, backend, dtype, device_id, ranges, *args
-        return (None, None, None, None, None, None, None, *grads)
+        # Grads wrt. formula, aliases, backend, dtype, device_id, ranges, optional_flags, rec_multVar_highdim, *args
+        return (None, None, None, None, None, None, None, None, *grads)
 
 
 class Genred():
@@ -178,7 +196,8 @@ class Genred():
         """
     
     def __init__(self, formula, aliases, reduction_op='Sum', axis=0, dtype=default_dtype, opt_arg=None,
-                 formula2=None, cuda_type=None, dtype_acc="auto", use_double_acc=False, sum_scheme="auto"):
+                 formula2=None, cuda_type=None, dtype_acc="auto", use_double_acc=False, sum_scheme="auto", 
+                 enable_chunks=True, optional_flags=[], rec_multVar_highdim=None):
         r"""
         Instantiate a new generic operation.
 
@@ -248,6 +267,11 @@ class Genred():
                     in the output. This improves accuracy for large sized data. 
                   - **sum_scheme** =  ``"kahan_scheme"``: use Kahan summation algorithm to compensate for round-off errors. This improves
                 accuracy for large sized data. 
+                
+            enable_chunks (bool, default True): enable automatic selection of special "chunked" computation mode for accelerating reductions
+				with formulas involving large dimension variables.
+				
+			optional_flags (list, default []): further optional flags passed to the compiler, in the form ['-D...=...','-D...=...']
 
         """
         if cuda_type:
@@ -256,8 +280,8 @@ class Genred():
         self.reduction_op = reduction_op
         reduction_op_internal, formula2 = preprocess(reduction_op, formula2)
         
-        self.accuracy_flags = get_accuracy_flags(dtype_acc, use_double_acc, sum_scheme, dtype, reduction_op_internal)
-
+        self.optional_flags = optional_flags + get_optional_flags(reduction_op_internal, dtype_acc, use_double_acc, sum_scheme, dtype, enable_chunks)
+        
         str_opt_arg = ',' + str(opt_arg) if opt_arg else ''
         str_formula2 = ',' + formula2 if formula2 else ''
         
@@ -267,6 +291,8 @@ class Genred():
         self.dtype = dtype
         self.axis = axis
         self.opt_arg = opt_arg
+        
+        self.rec_multVar_highdim = rec_multVar_highdim
 
     def __call__(self, *args, backend='auto', device_id=-1, ranges=None):
         r"""
@@ -393,7 +419,7 @@ class Genred():
             args, ranges, tag_dummy, N = preprocess_half2(args, self.aliases, self.axis, ranges, nx, ny)
         
         out = GenredAutograd.apply(self.formula, self.aliases, backend, self.dtype, 
-                                   device_id, ranges, self.accuracy_flags, *args)
+                                   device_id, ranges, self.optional_flags, self.rec_multVar_highdim, *args)
 
         if self.dtype in ('float16','half'):
             out = postprocess_half2(out, tag_dummy, self.reduction_op, N)
