@@ -3,7 +3,7 @@ import torch
 from pykeops.common.get_options import get_tag_backend
 from pykeops.common.keops_io import LoadKeOps
 from pykeops.common.operations import ConjugateGradientSolver
-from pykeops.common.parse_type import get_type, complete_aliases, get_accuracy_flags
+from pykeops.common.parse_type import get_type, complete_aliases, get_optional_flags
 from pykeops.common.utils import axis2cat
 from pykeops.torch import default_dtype
 from pykeops.torch import include_dirs
@@ -16,10 +16,19 @@ class KernelSolveAutograd(torch.autograd.Function):
     """
 
     @staticmethod
-    def forward(ctx, formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, accuracy_flags, *args):
+    def forward(ctx, formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, optional_flags, rec_multVar_highdim, *args):
     
-        optional_flags = include_dirs + accuracy_flags
+        optional_flags += include_dirs
 
+        # N.B. when rec_multVar_highdim option is set, it means that formula is of the form "sum(F*b)", where b is a variable
+        # with large dimension. In this case we set compiler option MULT_VAR_HIGHDIM to allow for the use of the special "final chunk" computation
+        # mode. However, this may not be also true for the gradients of the same formula. In fact only the gradient
+        # with respect to variable b will have the same form. Hence, we save optional_flags current status into ctx, 
+        # before adding the MULT_VAR_HIGHDIM compiler option.
+        ctx.optional_flags = optional_flags.copy()
+        if rec_multVar_highdim is not None:
+            optional_flags += ['-DMULT_VAR_HIGHDIM=1']
+        
         myconv = LoadKeOps(formula, aliases, dtype, 'torch',
                            optional_flags).import_module()
 
@@ -34,7 +43,8 @@ class KernelSolveAutograd(torch.autograd.Function):
         ctx.eps = eps
         ctx.myconv = myconv
         ctx.ranges = ranges
-        ctx.accuracy_flags = accuracy_flags
+        ctx.rec_multVar_highdim = rec_multVar_highdim
+        ctx.optional_flags = optional_flags
         if ranges is None: ranges = () # To keep the same type
             
         varinv = args[varinvpos]
@@ -42,17 +52,17 @@ class KernelSolveAutograd(torch.autograd.Function):
 
         tagCPUGPU, tag1D2D, tagHostDevice = get_tag_backend(backend, args)
 
-        if tagCPUGPU==1 & tagHostDevice==1:
+        if tagCPUGPU == 1 & tagHostDevice == 1:
             device_id = args[0].device.index
-            for i in range(1,len(args)):
+            for i in range(1, len(args)):
                 if args[i].device.index != device_id:
                     raise ValueError("[KeOps] Input arrays must be all located on the same device.")
 
         def linop(var):
-            newargs = args[:varinvpos] + (var,) + args[varinvpos+1:]
+            newargs = args[:varinvpos] + (var,) + args[varinvpos + 1:]
             res = myconv.genred_pytorch(tagCPUGPU, tag1D2D, tagHostDevice, device_id, ranges, *newargs)
             if alpha:
-                res += alpha*var
+                res += alpha * var
             return res
 
         global copy
@@ -76,7 +86,8 @@ class KernelSolveAutograd(torch.autograd.Function):
         eps = ctx.eps
         myconv = ctx.myconv
         ranges = ctx.ranges
-        accuracy_flags = ctx.accuracy_flags
+        optional_flags = ctx.optional_flags
+        rec_multVar_highdim = ctx.rec_multVar_highdim
 
         args = ctx.saved_tensors[:-1]  # Unwrap the saved variables
         nargs = len(args)
@@ -86,18 +97,18 @@ class KernelSolveAutograd(torch.autograd.Function):
         # wrt. the output, G, should be given as a 6-th variable (numbered 5),
         # with the same dim-cat as the formula's output.
         eta = 'Var(' + str(nargs) + ',' + str(myconv.dimout) + ',' + str(myconv.tagIJ) + ')'
-      
+
         # there is also a new variable for the formula's output
         resvar = 'Var(' + str(nargs+1) + ',' + str(myconv.dimout) + ',' + str(myconv.tagIJ) + ')'
         
         newargs = args[:varinvpos] + (G,) + args[varinvpos+1:]
-        KinvG = KernelSolveAutograd.apply(formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, accuracy_flags, *newargs)
+        KinvG = KernelSolveAutograd.apply(formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, optional_flags, rec_multVar_highdim, *newargs)
 
         grads = []  # list of gradients wrt. args;
 
         for (var_ind, sig) in enumerate(aliases):  # Run through the arguments
             # If the current gradient is to be discarded immediatly...
-            if not ctx.needs_input_grad[var_ind + 10]:  # because of (formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, accuracy_flags)
+            if not ctx.needs_input_grad[var_ind + 11]:  # because of (formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, optional_flags, rec_multVar_highdim)
                 grads.append(None)  # Don't waste time computing it.
 
             else:  # Otherwise, the current gradient is really needed by the user:
@@ -122,7 +133,7 @@ class KernelSolveAutograd(torch.autograd.Function):
                     if cat == 2:  # we're referring to a parameter, so we'll have to sum both wrt 'i' and 'j'
                         # WARNING !! : here we rely on the implementation of DiffT in files in folder keops/core/formulas/reductions
                         # if tagI==cat of V is 2, then reduction is done wrt j, so we need to further sum output wrt i
-                        grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, accuracy_flags, *args_g)
+                        grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, optional_flags, None, *args_g)
                         # Then, sum 'grad' wrt 'i' :
                         # I think that '.sum''s backward introduces non-contiguous arrays,
                         # and is thus non-compatible with GenredAutograd: grad = grad.sum(0)
@@ -130,12 +141,11 @@ class KernelSolveAutograd(torch.autograd.Function):
                         grad = torch.ones(1, grad.shape[0]).type_as(grad.data) @ grad
                         grad = grad.view(-1)
                     else:
-                        grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, accuracy_flags, *args_g)
+                        grad = genconv(formula_g, aliases_g, backend, dtype, device_id, ranges, optional_flags, None, *args_g)
                     grads.append(grad)
          
-        # Grads wrt. formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, accuracy_flags, *args
-        return (None, None, None, None, None, None, None, None, None, None, *grads)
-
+        # Grads wrt. formula, aliases, varinvpos, alpha, backend, dtype, device_id, eps, ranges, optional_flags, rec_multVar_highdim, *args
+        return (None, None, None, None, None, None, None, None, None, None, None, *grads)
 
 
 class KernelSolve():
@@ -180,7 +190,7 @@ class KernelSolve():
         >>> print(g_x.shape)
         torch.Size([10000, 3]) 
     """
-    def __init__(self, formula, aliases, varinvalias, axis=0, dtype=default_dtype, cuda_type=None, dtype_acc="auto", use_double_acc=False, sum_scheme="auto"):
+    def __init__(self, formula, aliases, varinvalias, axis=0, dtype=default_dtype, cuda_type=None, dtype_acc="auto", use_double_acc=False, sum_scheme="auto", enable_chunks=True, rec_multVar_highdim=None):
         r"""
         Instantiate a new KernelSolve operation.
 
@@ -252,13 +262,16 @@ class KernelSolve():
                   - **sum_scheme** =  ``"kahan_scheme"``: use Kahan summation algorithm to compensate for round-off errors. This improves
                 accuracy for large sized data. 
 
+            enable_chunks (bool, default True): enable automatic selection of special "chunked" computation mode for accelerating reductions
+				with formulas involving large dimension variables.
+
         """
         if cuda_type:
             # cuda_type is just old keyword for dtype, so this is just a trick to keep backward compatibility
-            dtype = cuda_type 
+            dtype = cuda_type
         reduction_op = 'Sum'
 
-        self.accuracy_flags = get_accuracy_flags(dtype_acc, use_double_acc, sum_scheme, dtype, reduction_op)
+        self.optional_flags = get_optional_flags(reduction_op, dtype_acc, use_double_acc, sum_scheme, dtype, enable_chunks)
 
         self.formula = reduction_op + '_Reduction(' + formula + ',' + str(axis2cat(axis)) + ')'
         self.aliases = complete_aliases(formula, list(aliases))  # just in case the user provided a tuple
@@ -273,6 +286,7 @@ class KernelSolve():
             varinvpos = tmp.index(varinvalias)
         self.varinvpos = varinvpos
         self.dtype = dtype
+        self.rec_multVar_highdim = rec_multVar_highdim
 
     def __call__(self, *args, backend='auto', device_id=-1, alpha=1e-10, eps=1e-6, ranges=None):
         r"""
@@ -326,7 +340,4 @@ class KernelSolve():
             
         """
 
-        return KernelSolveAutograd.apply(self.formula, self.aliases, self.varinvpos, alpha, backend, self.dtype, device_id, eps, ranges, self.accuracy_flags, *args)
-
-
-
+        return KernelSolveAutograd.apply(self.formula, self.aliases, self.varinvpos, alpha, backend, self.dtype, device_id, eps, ranges, self.optional_flags, self.rec_multVar_highdim, *args)
